@@ -54,6 +54,24 @@ typedef enum
 	 GPM_INFO_ERROR_DATA_NOT_AVAILABLE
 } GpmInfoError;
 
+typedef enum {
+	GPM_EVENT_ON_AC,
+	GPM_EVENT_ON_BATTERY,
+	GPM_EVENT_SESSION_IDLE,
+	GPM_EVENT_SESSION_ACTIVE,
+	GPM_EVENT_SUSPEND,
+	GPM_EVENT_HIBERNATE,
+	GPM_EVENT_RESUME,
+	GPM_EVENT_LID_CLOSED,
+	GPM_EVENT_LID_OPENED,
+	GPM_EVENT_NOTIFICATION,
+	GPM_EVENT_DPMS_ON,
+	GPM_EVENT_DPMS_STANDBY,
+	GPM_EVENT_DPMS_SUSPEND,
+	GPM_EVENT_DPMS_OFF,
+	GPM_EVENT_LAST
+} GpmGraphWidgetEvent;
+
 #define GPM_INFO_ERROR gpm_manager_error_quark ()
 
 GType		 gpm_info_get_type			(void);
@@ -82,6 +100,10 @@ gboolean	 gpm_statistics_get_axis_type		(GpmInfo	*info,
 							 gchar		**axis_type_x,
 							 gchar		**axis_type_y,
 							 GError		**error);
+void		 gpm_info_explain_reason		(GpmInfo	*info,
+							 GpmGraphWidgetEvent event,
+							 const gchar	*pre,
+							 const gchar	*post);
 
 G_END_DECLS
 
