@@ -42,15 +42,18 @@
 
 #include <libgnomeui/gnome-help.h>
 
-#ifdef HAVE_LIBNOTIFY
-#include <libnotify/notify.h>
-#endif
-
+#include "gpm-ac-adapter.h"
+#include "gpm-battery.h"
+#include "gpm-conf.h"
+#include "gpm-control.h"
 #include "gpm-common.h"
-#include "gpm-stock-icons.h"
-#include "gpm-power.h"
-#include "gpm-tray-icon.h"
 #include "gpm-debug.h"
+#include "gpm-hal.h"
+#include "gpm-notify.h"
+#include "gpm-control.h"
+#include "gpm-power.h"
+#include "gpm-stock-icons.h"
+#include "gpm-tray-icon.h"
 
 static void     gpm_tray_icon_class_init (GpmTrayIconClass *klass);
 static void     gpm_tray_icon_init       (GpmTrayIcon      *tray_icon);
@@ -62,21 +65,27 @@ static void     gpm_tray_icon_finalize   (GObject	   *object);
 
 struct GpmTrayIconPrivate
 {
-	GtkStatusIcon		*status_icon;
+	GpmAcAdapter		*ac_adapter;
+	GpmConf			*conf;
+	GpmHal			*hal;
+	GpmControl		*control;
 	GpmPower		*power;
-	gboolean		 show_notifications;
+	GpmBattery		*battery;
+	GpmNotify		*notify;
+
+	GtkStatusIcon		*status_icon;
+	guint			 low_percentage;
 	gboolean		 is_visible;
 	gboolean		 show_suspend;
 	gboolean		 show_hibernate;
 	gchar			*stock_id;
-#ifdef HAVE_LIBNOTIFY
-	NotifyNotification	*notify;
-#endif
 };
 
 enum {
 	SUSPEND,
 	HIBERNATE,
+	DESCRIPTION_CHANGED,
+	ICON_CHANGED,
 	LAST_SIGNAL
 };
 
@@ -86,13 +95,6 @@ enum {
 };
 
 static guint	 signals [LAST_SIGNAL] = { 0, };
-static gboolean
-libnotify_event (GpmTrayIcon    *icon,
-		 const gchar	*title,
-		 const gchar	*content,
-		 guint		 timeout,
-		 const gchar	*msgicon,
-		 GpmNotifyLevel	 urgency);
 
 G_DEFINE_TYPE (GpmTrayIcon, gpm_tray_icon, G_TYPE_OBJECT)
 
@@ -101,7 +103,7 @@ G_DEFINE_TYPE (GpmTrayIcon, gpm_tray_icon, G_TYPE_OBJECT)
  * @icon: This TrayIcon class instance
  * @enabled: If we should enable (i.e. show) the suspend icon
  **/
-void
+static void
 gpm_tray_icon_enable_suspend (GpmTrayIcon *icon,
 			      gboolean     enabled)
 {
@@ -114,7 +116,7 @@ gpm_tray_icon_enable_suspend (GpmTrayIcon *icon,
  * @icon: This TrayIcon class instance
  * @enabled: If we should enable (i.e. show) the hibernate icon
  **/
-void
+static void
 gpm_tray_icon_enable_hibernate (GpmTrayIcon *icon,
 				gboolean     enabled)
 {
@@ -150,6 +152,9 @@ gpm_tray_icon_set_image_from_stock (GpmTrayIcon *icon,
 {
 	g_return_if_fail (GPM_IS_TRAY_ICON (icon));
 
+	gpm_debug ("emitting icon-changed");
+	g_signal_emit (icon, signals [ICON_CHANGED], 0);
+
 	if (stock_id) {
 		/* we only set a new icon if the name differs */
 		if (strcmp (icon->priv->stock_id, stock_id) != 0) {
@@ -198,7 +203,9 @@ gpm_tray_icon_show_info_cb (GtkMenuItem *item, gpointer data)
 	longdesc = g_strdup (gdesc->str);
 	g_string_free (gdesc, TRUE);
 
-	libnotify_event (icon, desc, longdesc, 0, msgicon, GPM_NOTIFY_URGENCY_LOW);
+	gpm_notify_display (icon->priv->notify, desc, longdesc,
+			    GPM_NOTIFY_TIMEOUT_NEVER, msgicon,
+			    GPM_NOTIFY_URGENCY_LOW);
 }
 
 /**
@@ -349,7 +356,7 @@ gpm_tray_icon_popup_cleared_cd (GtkWidget   *widget,
 				GpmTrayIcon *icon)
 {
 //	g_return_if_fail (GPM_IS_TRAY_ICON (icon));
-	g_debug ("clear tray (icon = %p)", icon);
+	gpm_debug ("clear tray (icon = %p)", icon);
 }
 
 /**
@@ -407,6 +414,26 @@ gpm_tray_icon_class_init (GpmTrayIconClass *klass)
 			      g_cclosure_marshal_VOID__VOID,
 			      G_TYPE_NONE,
 			      0);
+	signals [DESCRIPTION_CHANGED] =
+		g_signal_new ("description-changed",
+			      G_TYPE_FROM_CLASS (object_class),
+			      G_SIGNAL_RUN_LAST,
+			      G_STRUCT_OFFSET (GpmTrayIconClass, description_changed),
+			      NULL,
+			      NULL,
+			      g_cclosure_marshal_VOID__STRING,
+			      G_TYPE_NONE,
+			      1, G_TYPE_STRING);
+	signals [ICON_CHANGED] =
+		g_signal_new ("icon-changed",
+			      G_TYPE_FROM_CLASS (object_class),
+			      G_SIGNAL_RUN_LAST,
+			      G_STRUCT_OFFSET (GpmTrayIconClass, icon_changed),
+			      NULL,
+			      NULL,
+			      g_cclosure_marshal_VOID__STRING,
+			      G_TYPE_NONE,
+			      1, G_TYPE_STRING);
 }
 
 /**
@@ -414,19 +441,14 @@ gpm_tray_icon_class_init (GpmTrayIconClass *klass)
  * @icon: This TrayIcon class instance
  * @enabled: If we should show the tray
  **/
-void
+static void
 gpm_tray_icon_show (GpmTrayIcon *icon,
 		    gboolean     enabled)
 {
 	g_return_if_fail (GPM_IS_TRAY_ICON (icon));
 
-	if (enabled) {
-		gtk_status_icon_set_visible (GTK_STATUS_ICON (icon->priv->status_icon), TRUE);
-		icon->priv->is_visible = TRUE;
-	} else {
-		gtk_status_icon_set_visible (GTK_STATUS_ICON (icon->priv->status_icon), FALSE);
-		icon->priv->is_visible = FALSE;
-	}
+	gtk_status_icon_set_visible (GTK_STATUS_ICON (icon->priv->status_icon), enabled);
+	icon->priv->is_visible = enabled != FALSE;
 }
 
 /**
@@ -436,6 +458,8 @@ gpm_tray_icon_show (GpmTrayIcon *icon,
  **/
 static void
 gpm_tray_icon_popup_menu_cb (GtkStatusIcon *status_icon,
+			     guint          button,
+			     guint32        timestamp,
 			     GpmTrayIcon   *icon)
 {
 	GtkMenu *menu = (GtkMenu*) gtk_menu_new ();
@@ -484,7 +508,10 @@ gpm_tray_icon_popup_menu_cb (GtkStatusIcon *status_icon,
 	gtk_widget_show_all (GTK_WIDGET (menu));
 	gtk_menu_popup (GTK_MENU (menu), NULL, NULL,
 			gtk_status_icon_position_menu, status_icon,
-			1, gtk_get_current_event_time());
+			button, timestamp);
+	if (button == 0) {
+		gtk_menu_shell_select_first (GTK_MENU_SHELL (menu), FALSE);
+	}
 
 	g_signal_connect (GTK_WIDGET (menu), "hide",
 			  G_CALLBACK (gpm_tray_icon_popup_cleared_cd), icon);
@@ -604,24 +631,364 @@ gpm_tray_icon_activate_cb (GtkStatusIcon *status_icon,
 			  G_CALLBACK (gpm_tray_icon_popup_cleared_cd), icon);
 }
 
+/**
+ * get_stock_id:
+ * @icon: This class instance
+ * @icon_policy: The policy set from gconf
+ *
+ * Get the stock filename id after analysing the state of all the devices
+ * attached to the computer, and applying policy from gconf.
+ *
+ * Return value: The icon filename, must free using g_free.
+ **/
+static char *
+get_stock_id (GpmTrayIcon *icon,
+	      guint	   icon_policy)
+{
+	GpmPowerStatus status_primary;
+	GpmPowerStatus status_ups;
+	GpmPowerStatus status_mouse;
+	GpmPowerStatus status_keyboard;
+	gboolean present;
+
+	if (icon_policy == GPM_ICON_POLICY_NEVER) {
+		gpm_debug ("The key " GPM_CONF_ICON_POLICY
+			   " is set to never, so no icon will be displayed.\n"
+			   "You can change this using gnome-power-preferences");
+		return NULL;
+	}
+
+	/* Finds if a device was found in the cache AND that it is present */
+	present = gpm_power_get_battery_status (icon->priv->power,
+						GPM_POWER_KIND_PRIMARY,
+						&status_primary);
+	status_primary.is_present &= present;
+	present = gpm_power_get_battery_status (icon->priv->power,
+						GPM_POWER_KIND_UPS,
+						&status_ups);
+	status_ups.is_present &= present;
+	present = gpm_power_get_battery_status (icon->priv->power,
+						GPM_POWER_KIND_MOUSE,
+						&status_mouse);
+	status_mouse.is_present &= present;
+	present = gpm_power_get_battery_status (icon->priv->power,
+						GPM_POWER_KIND_KEYBOARD,
+						&status_keyboard);
+	status_keyboard.is_present &= present;
+
+	/* we try CRITICAL: PRIMARY, UPS, MOUSE, KEYBOARD */
+	gpm_debug ("Trying CRITICAL icon: primary, ups, mouse, keyboard");
+	if (status_primary.is_present &&
+	    status_primary.percentage_charge < icon->priv->low_percentage) {
+		return gpm_power_get_icon_from_status (&status_primary, GPM_POWER_KIND_PRIMARY);
+
+	} else if (status_ups.is_present &&
+		   status_ups.percentage_charge < icon->priv->low_percentage) {
+		return gpm_power_get_icon_from_status (&status_ups, GPM_POWER_KIND_UPS);
+
+	} else if (status_mouse.is_present &&
+		   status_mouse.percentage_charge < icon->priv->low_percentage) {
+		return gpm_power_get_icon_from_status (&status_ups, GPM_POWER_KIND_MOUSE);
+
+	} else if (status_keyboard.is_present &&
+		   status_keyboard.percentage_charge < icon->priv->low_percentage) {
+		return gpm_power_get_icon_from_status (&status_ups, GPM_POWER_KIND_KEYBOARD);
+	}
+
+	if (icon_policy == GPM_ICON_POLICY_CRITICAL) {
+		gpm_debug ("no devices critical, so no icon will be displayed.");
+		return NULL;
+	}
+
+	/* we try (DIS)CHARGING: PRIMARY, UPS */
+	gpm_debug ("Trying CHARGING icon: primary, ups");
+	if (status_primary.is_present &&
+	    (status_primary.is_charging || status_primary.is_discharging) ) {
+		return gpm_power_get_icon_from_status (&status_primary, GPM_POWER_KIND_PRIMARY);
+
+	} else if (status_ups.is_present &&
+		   (status_ups.is_charging || status_ups.is_discharging) ) {
+		return gpm_power_get_icon_from_status (&status_ups, GPM_POWER_KIND_UPS);
+	}
+
+	/* Check if we should just show the icon all the time */
+	if (icon_policy == GPM_ICON_POLICY_CHARGE) {
+		gpm_debug ("no devices (dis)charging, so no icon will be displayed.");
+		return NULL;
+	}
+
+	/* we try PRESENT: PRIMARY, UPS */
+	gpm_debug ("Trying PRESENT icon: primary, ups");
+	if (status_primary.is_present) {
+		return gpm_power_get_icon_from_status (&status_primary, GPM_POWER_KIND_PRIMARY);
+
+	} else if (status_ups.is_present) {
+		return gpm_power_get_icon_from_status (&status_ups, GPM_POWER_KIND_UPS);
+	}
+
+	/* Check if we should just fallback to the ac icon */
+	if (icon_policy == GPM_ICON_POLICY_PRESENT) {
+		gpm_debug ("no devices present, so no icon will be displayed.");
+		return NULL;
+	}
+
+	/* we fallback to the ac_adapter icon */
+	gpm_debug ("Using fallback");
+	return g_strdup_printf (GPM_STOCK_AC_ADAPTER);
+}
+
+/**
+ * gpm_tray_icon_sync:
+ * @icon: This class instance
+ *
+ * Update the tray icon and set the correct tooltip when required, or remove
+ * (hide) the icon when no longer required by policy.
+ **/
+void
+gpm_tray_icon_sync (GpmTrayIcon *icon)
+{
+	gchar *stock_id = NULL;
+	gchar *icon_policy_str;
+	gint   icon_policy;
+
+	/* do we want to display the icon */
+	gpm_conf_get_string (icon->priv->conf, GPM_CONF_ICON_POLICY, &icon_policy_str);
+	icon_policy = gpm_tray_icon_mode_from_string (icon_policy_str);
+
+	g_free (icon_policy_str);
+
+	/* try to get stock image */
+	stock_id = get_stock_id (icon, icon_policy);
+
+	gpm_debug ("Going to use stock id: %s", stock_id);
+
+	/* only create if we have a valid filename */
+	if (stock_id) {
+		gchar *tooltip = NULL;
+
+		gpm_tray_icon_set_image_from_stock (icon, stock_id);
+		/* make sure that we are visible */
+		gpm_tray_icon_show (icon, TRUE);
+		g_free (stock_id);
+
+		gpm_debug ("emitting description-changed");
+		g_signal_emit (icon, signals [DESCRIPTION_CHANGED], 0);
+
+		gpm_power_get_status_summary (icon->priv->power, &tooltip, NULL);
+
+		gpm_tray_icon_set_tooltip (icon, tooltip);
+		g_free (tooltip);
+	} else {
+		/* remove icon */
+		gpm_debug ("no icon will be displayed");
+
+		/* make sure that we are hidden */
+		gpm_tray_icon_show (icon, FALSE);
+	}
+}
+
+gboolean
+gpm_ui_get_description (GpmTrayIcon  *icon,
+		        gchar       **description,
+		        GError      **error)
+{
+	gpm_debug ("org.gnome.PowerManager.UI.GetDescription");
+
+	/* return tooltip */
+	gpm_power_get_status_summary (icon->priv->power, description, NULL);
+	return TRUE;
+}
+
+gboolean
+gpm_ui_get_icon (GpmTrayIcon  *icon,
+		 gchar       **iconname,
+		 GError      **error)
+{
+	gchar *icon_policy_str;
+	gint   icon_policy;
+
+	/* do we want to display the icon */
+	gpm_conf_get_string (icon->priv->conf, GPM_CONF_ICON_POLICY, &icon_policy_str);
+	icon_policy = gpm_tray_icon_mode_from_string (icon_policy_str);
+
+	g_free (icon_policy_str);
+
+	gpm_debug ("org.gnome.PowerManager.UI.GetIcon");
+
+	/* return icon name */
+	*iconname = get_stock_id (icon, icon_policy);
+	return TRUE;
+}
+
+/**
+ * power_on_ac_changed_cb:
+ * @power: The power class instance
+ * @on_ac: if we are on AC power
+ * @icon: This class instance
+ *
+ * Does the actions when the ac power source is inserted/removed.
+ **/
+static void
+ac_adapter_changed_cb (GpmAcAdapter     *ac_adapter,
+		       GpmAcAdapterState state,
+		       GpmTrayIcon      *icon)
+{
+	gpm_tray_icon_sync (icon);
+}
+
+/**
+ * conf_key_changed_cb:
+ *
+ * We might have to do things when the gconf keys change; do them here.
+ **/
+static void
+conf_key_changed_cb (GpmConf     *conf,
+		     const gchar *key,
+		     GpmTrayIcon *icon)
+{
+	gboolean    enabled;
+	gboolean    allowed_in_menu;
+
+	if (strcmp (key, GPM_CONF_ICON_POLICY) == 0) {
+		gpm_tray_icon_sync (icon);
+
+	} else if (strcmp (key, GPM_CONF_CAN_SUSPEND) == 0) {
+		gpm_control_allowed_suspend (icon->priv->control, &enabled, NULL);
+		gpm_conf_get_bool (icon->priv->conf, GPM_CONF_SHOW_ACTIONS_IN_MENU, &allowed_in_menu);
+		gpm_tray_icon_enable_suspend (icon, allowed_in_menu && enabled);
+
+	} else if (strcmp (key, GPM_CONF_CAN_HIBERNATE) == 0) {
+		gpm_control_allowed_hibernate (icon->priv->control, &enabled, NULL);
+		gpm_conf_get_bool (icon->priv->conf, GPM_CONF_SHOW_ACTIONS_IN_MENU, &allowed_in_menu);
+		gpm_tray_icon_enable_hibernate (icon, allowed_in_menu && enabled);
+
+	} else if (strcmp (key, GPM_CONF_SHOW_ACTIONS_IN_MENU) == 0) {
+		gpm_conf_get_bool (icon->priv->conf, GPM_CONF_SHOW_ACTIONS_IN_MENU, &allowed_in_menu);
+		gpm_control_allowed_suspend (icon->priv->control, &enabled, NULL);
+		gpm_tray_icon_enable_suspend (icon, allowed_in_menu && enabled);
+		gpm_control_allowed_hibernate (icon->priv->control, &enabled, NULL);
+		gpm_tray_icon_enable_hibernate (icon, allowed_in_menu && enabled);
+	}
+}
+
+/**
+ * battery_removed_cb:
+ * @battery: The battery class
+ * @udi: The HAL udi of the device that was removed
+ * @manager: This class instance
+ **/
+static void
+battery_removed_cb (GpmBattery *battery,
+		    const gchar *udi,
+		    GpmTrayIcon *icon)
+{
+	gpm_debug ("Battery Removed: %s", udi);
+	gpm_tray_icon_sync (icon);
+}
+
+/**
+ * hal_daemon_monitor_cb:
+ * @hal: The HAL class instance
+ **/
+static void
+hal_daemon_monitor_cb (GpmHal      *hal,
+		       GpmTrayIcon *icon)
+{
+	gpm_tray_icon_sync (icon);
+}
+
+/**
+ * control_sleep_failure_cb:
+ * @control: The control class instance
+ * @icon: This icon class instance
+ *
+ * We have to notify the user is suspend failed
+ **/
+static void
+control_sleep_failure_cb (GpmControl  *control,
+			  GpmControlAction action,
+		          GpmTrayIcon *icon)
+{
+	gchar *message;
+	gboolean show_notify;
+	gchar *title;
+
+	/* We only show the HAL failed notification if set in gconf */
+	gpm_conf_get_bool (icon->priv->conf, GPM_CONF_NOTIFY_HAL_ERROR, &show_notify);
+	if (show_notify) {
+		if (action == GPM_CONTROL_ACTION_HIBERNATE) {
+			title = _("Hibernate Problem");
+			message = g_strdup_printf (_("HAL failed to %s. "
+						     "Check the help file for common problems."),
+						     _("hibernate"));
+			gpm_syslog ("hibernate failed");
+		} else {
+			message = g_strdup_printf (_("Your computer failed to %s.\n"
+						     "Check the help file for common problems."),
+						     _("suspend"));
+			title = _("Suspend Problem");
+			gpm_syslog ("suspend failed");
+		}
+		gpm_notify_display (icon->priv->notify,
+				    title, message,
+				    GPM_NOTIFY_TIMEOUT_LONG,
+				    GTK_STOCK_DIALOG_WARNING,
+				    GPM_NOTIFY_URGENCY_LOW);
+		g_free (message);
+	}
+}
+
 /**
  * gpm_tray_icon_init:
  * @icon: This TrayIcon class instance
  *
- * Initialise the tray object, and set up libnotify
+ * Initialise the tray object
  **/
 static void
 gpm_tray_icon_init (GpmTrayIcon *icon)
 {
-	gboolean ret = TRUE;
+	gboolean enabled;
+	gboolean allowed_in_menu;
 
 	icon->priv = GPM_TRAY_ICON_GET_PRIVATE (icon);
 
-	/* FIXME: make this a property */
-	icon->priv->show_notifications = TRUE;
 	icon->priv->stock_id = g_strdup ("about-blank");
+
+	/* we use power for the messages and the icon state */
 	icon->priv->power = gpm_power_new ();
 
+	/* use libnotify */
+	icon->priv->notify = gpm_notify_new ();
+
+	/* use the policy object */
+	icon->priv->control = gpm_control_new ();
+	g_signal_connect (icon->priv->control, "sleep-failure",
+			  G_CALLBACK (control_sleep_failure_cb), icon);
+
+	icon->priv->battery = gpm_battery_new ();
+	/* we need these to refresh the tooltip and icon */
+	g_signal_connect (icon->priv->battery, "battery-removed",
+			  G_CALLBACK (battery_removed_cb), icon);
+
+	/* we need this to refresh the tooltip and icon on hal restart */
+	icon->priv->hal = gpm_hal_new ();
+	g_signal_connect (icon->priv->hal, "daemon-start",
+			  G_CALLBACK (hal_daemon_monitor_cb), icon);
+	g_signal_connect (icon->priv->hal, "daemon-stop",
+			  G_CALLBACK (hal_daemon_monitor_cb), icon);
+
+	icon->priv->conf = gpm_conf_new ();
+	g_signal_connect (icon->priv->conf, "value-changed",
+			  G_CALLBACK (conf_key_changed_cb), icon);
+
+	/* get percentage policy */
+	gpm_conf_get_uint (icon->priv->conf, GPM_CONF_LOW_PERCENTAGE, &icon->priv->low_percentage);
+
+	/* we use ac_adapter so we can log the event */
+	icon->priv->ac_adapter = gpm_ac_adapter_new ();
+	g_signal_connect (icon->priv->ac_adapter, "ac-adapter-changed",
+			  G_CALLBACK (ac_adapter_changed_cb), icon);
+
 	icon->priv->status_icon = gtk_status_icon_new ();
 	g_signal_connect_object (G_OBJECT (icon->priv->status_icon),
 				 "popup_menu",
@@ -631,15 +998,17 @@ gpm_tray_icon_init (GpmTrayIcon *icon)
 				 "activate",
 				 G_CALLBACK (gpm_tray_icon_activate_cb),
 				 icon, 0);
+	gpm_notify_use_status_icon (icon->priv->notify, icon->priv->status_icon);
+
+	/* only show the suspend and hibernate icons if we can do the action,
+	   and the policy allows the actions in the menu */
+	gpm_conf_get_bool (icon->priv->conf, GPM_CONF_SHOW_ACTIONS_IN_MENU, &allowed_in_menu);
+	gpm_control_allowed_suspend (icon->priv->control, &enabled, NULL);
+	gpm_tray_icon_enable_suspend (icon, enabled && allowed_in_menu);
+	gpm_control_allowed_hibernate (icon->priv->control, &enabled, NULL);
+	gpm_tray_icon_enable_hibernate (icon, enabled && allowed_in_menu);
 
 	gpm_tray_icon_show (GPM_TRAY_ICON (icon), FALSE);
-
-#ifdef HAVE_LIBNOTIFY
-	ret = notify_init (GPM_NAME);
-#endif
-	if (!ret) {
-		gpm_warning ("gpm_tray_icon_init failed");
-	}
 }
 
 /**
@@ -659,17 +1028,27 @@ gpm_tray_icon_finalize (GObject *object)
 	if (tray_icon->priv->stock_id != NULL) {
 		g_free (tray_icon->priv->stock_id);
 	}
+	if (tray_icon->priv->notify != NULL) {
+		g_object_unref (tray_icon->priv->notify);
+	}
+	if (tray_icon->priv->control != NULL) {
+		g_object_unref (tray_icon->priv->control);
+	}
 	if (tray_icon->priv->power != NULL) {
 		g_object_unref (tray_icon->priv->power);
 	}
 	if (tray_icon->priv->status_icon != NULL) {
 		g_object_unref (tray_icon->priv->status_icon);
 	}
-#ifdef HAVE_LIBNOTIFY
-	if (tray_icon->priv->notify != NULL) {
-		notify_notification_close (tray_icon->priv->notify, NULL);
+	if (tray_icon->priv->ac_adapter != NULL) {
+		g_object_unref (tray_icon->priv->ac_adapter);
+	}
+	if (tray_icon->priv->battery != NULL) {
+		g_object_unref (tray_icon->priv->battery);
+	}
+	if (tray_icon->priv->hal != NULL) {
+		g_object_unref (tray_icon->priv->hal);
 	}
-#endif
 
 	g_return_if_fail (tray_icon->priv != NULL);
 
@@ -687,175 +1066,3 @@ gpm_tray_icon_new (void)
 	tray_icon = g_object_new (GPM_TYPE_TRAY_ICON, NULL);
 	return GPM_TRAY_ICON (tray_icon);
 }
-
-#ifdef HAVE_LIBNOTIFY
-
-/**
- * notification_closed_cb:
- * @notify: our libnotify instance
- * @icon: This TrayIcon class instance
- **/
-static void
-notification_closed_cb (NotifyNotification *notify,
-			GpmTrayIcon	*icon)
-{
-	/* just invalidate the pointer */
-	gpm_debug ("caught notification closed signal");
-	icon->priv->notify = NULL;
-}
-
-/**
- * libnotify_event:
- * @icon: This icon class instance
- * @title: The title, e.g. "Battery Low"
- * @content: The contect, e.g. "17 minutes remaining"
- * @timeout: The time we should remain on screen in seconds
- * @msgicon: The icon to display, or NULL, e.g. GPM_STOCK_UPS_CHARGING_080
- * @urgency: The urgency type, e.g. GPM_NOTIFY_URGENCY_CRITICAL
- *
- * Does a libnotify messagebox dialogue.
- * Return value: success
- **/
-static gboolean
-libnotify_event (GpmTrayIcon    *icon,
-		 const gchar	*title,
-		 const gchar	*content,
-		 guint		 timeout,
-		 const gchar	*msgicon,
-		 GpmNotifyLevel	 urgency)
-{
-	if (icon->priv->notify != NULL) {
-		notify_notification_close (icon->priv->notify, NULL);
-	}
-
-/* DISTROS: If you've patched your libnotify 0.4.2 package you can remove this check */
-#if HAVE_LIBNOTIFY_NEW
-	icon->priv->notify = notify_notification_new_with_status_icon (title, content,
-								       msgicon, icon->priv->status_icon);
-#else
-	/* we can't point because of a bug in libnotify. Need dependency on 0.4.3 */
-	icon->priv->notify = notify_notification_new (title, content,
-						      msgicon, NULL);
-#endif
-
-	notify_notification_set_timeout (icon->priv->notify, timeout * 1000);
-
-	if (urgency == GPM_NOTIFY_URGENCY_CRITICAL) {
-		gpm_warning ("libnotify: %s : %s", GPM_NAME, content);
-	} else {
-		gpm_debug ("libnotify: %s : %s", GPM_NAME, content);
-	}
-
-	g_signal_connect (icon->priv->notify, "closed", G_CALLBACK (notification_closed_cb), icon);
-
-	if (! notify_notification_show (icon->priv->notify, NULL)) {
-		gpm_warning ("failed to send notification (%s)", content);
-		return FALSE;
-	}
-
-	return TRUE;
-}
-
-#else
-
-/**
- * libnotify_event:
- * @icon: This icon class instance
- * @title: The title, e.g. "Battery Low"
- * @content: The contect, e.g. "17 minutes remaining"
- * @timeout: The time we should remain on screen in seconds
- * @msgicon: The icon to display, or NULL, e.g. GPM_STOCK_UPS_CHARGING_080
- * @urgency: The urgency type, e.g. GPM_NOTIFY_URGENCY_CRITICAL
- *
- * Does a gtk messagebox dialogue.
- * Return value: success
- **/
-static gboolean
-libnotify_event (GpmTrayIcon    *icon,
-		 const gchar	*title,
-		 const gchar	*content,
-		 guint		 timeout,
-		 const gchar	*msgicon,
-		 GpmNotifyLevel	 urgency)
-{
-	GtkWidget     *dialog;
-	GtkMessageType msg_type;
-
-	if (urgency == GPM_NOTIFY_URGENCY_CRITICAL) {
-		msg_type = GTK_MESSAGE_WARNING;
-	} else {
-		msg_type = GTK_MESSAGE_INFO;
-	}
-
-	dialog = gtk_message_dialog_new_with_markup (NULL,
-						     GTK_DIALOG_DESTROY_WITH_PARENT,
-						     msg_type,
-						     GTK_BUTTONS_CLOSE,
-						     "<span size='larger'><b>%s</b></span>",
-						     GPM_NAME);
-
-	gtk_message_dialog_format_secondary_markup (GTK_MESSAGE_DIALOG (dialog), content);
-
-	g_signal_connect_swapped (dialog,
-				  "response",
-				  G_CALLBACK (gtk_widget_destroy),
-				  dialog);
-
-	gtk_window_present (GTK_WINDOW (dialog));
-
-	return TRUE;
-}
-#endif
-
-/**
- * gpm_tray_icon_notify:
- * @icon: This icon class instance
- * @title: The title, e.g. "Battery Low"
- * @content: The contect, e.g. "17 minutes remaining"
- * @timeout: The time we should remain on screen in seconds
- * @msgicon: The icon to display, or NULL, e.g. GPM_STOCK_UPS_CHARGING_080
- * @urgency: The urgency type, e.g. GPM_NOTIFY_URGENCY_CRITICAL
- *
- * Does a libnotify or gtk messagebox dialogue.
- **/
-void
-gpm_tray_icon_notify (GpmTrayIcon	*icon,
-		      const gchar	*title,
-		      const gchar	*content,
-		      guint		 timeout,
-		      const gchar	*msgicon,
-		      GpmNotifyLevel	 urgency)
-{
-	g_return_if_fail (GPM_IS_TRAY_ICON (icon));
-
-	if (! icon->priv->show_notifications) {
-		gpm_debug ("ignoring notification: %s", title);
-		return;
-	}
-
-	gpm_debug ("doing notify: %s", title);
-	libnotify_event (icon, title, content, timeout, msgicon, urgency);
-}
-
-/**
- * gpm_tray_icon_cancel_notify:
- * @icon: This icon class instance
- *
- * Cancels the notification, i.e. removes it from the screen.
- **/
-void
-gpm_tray_icon_cancel_notify (GpmTrayIcon *icon)
-{
-	GError *error;
-	g_return_if_fail (GPM_IS_TRAY_ICON (icon));
-	error = NULL;
-
-#ifdef HAVE_LIBNOTIFY
-	if (icon->priv->notify != NULL) {
-		g_object_unref (icon->priv->notify);
-	}
-	if (error != NULL) {
-		g_error_free (error);
-	}
-#endif
-}
